//+------------------------------------------------------------------+ 
//|                                               FILE_OPERATION.mqh |
//+------------------------------------------------------------------+ 
//+------------------------------------------------------------------+ 
//| CHECKING IF THE DIRECTORY EXISTS AND CREATING THE DIRECTORY IF IT DOES NOT EXIST                       |
//+------------------------------------------------------------------+ 
string CheckCreateGetPath()
  {
   long search=-1;
   string ffname="",lpath="",folder="*";
   string root=NAME_EXPERT+"\\"; // Root folder of the Expert Advisor
   bool flgROOT=false; // Flag for existence/non-existence of the root folder of the Expert Advisor
//+------------------------------------------------------------------+ 
//| SEARCHING FOR THE ROOT FOLDER OF THE EXPERT ADVISOR              |
//+------------------------------------------------------------------+ 
   lpath=folder;
   search=FileFindFirst(lpath,ffname); // Set the search handle in the MetaTrader 5\MQL5\Files folder
//---
// If the first folder is the root folder, flag it
   if(ffname==root)
     { flgROOT=true; /*Print("The root folder of the Expert Advisor "+root+" exists");*/ }
//---
   if(search!=INVALID_HANDLE) // If the search handle has been obtained
     {
      if(!flgROOT) // If the first folder is not the root folder
        {
         // search through all files to find the root folder
         while(FileFindNext(search,ffname))
           {
            if(IsStopped()) // The search has been stopped by the user
              { Print("------\nThe user has deleted the Expert Advisor!"); return(""); }
            //---
            if(ffname==root) // If the root folder is found, flag it
              { flgROOT=true; /*Print("The root folder "+root+" exists");*/ break; }
           }
        }
      //---
      FileFindClose(search); search=-1; // Close the root folder search handle
     }
   else { Print("Error when obtaining the search handle or the directory "+TRM_DP+" is empty: ",ErrorDesc(Error())); }
//+------------------------------------------------------------------+ 
//| CREATING THE REQUIRED DIRECTORY BASED ON SEARCH RESULTS          |
//+------------------------------------------------------------------+ 
   if(!flgROOT) // If the root folder does not exist
     {
      if(FolderCreate(NAME_EXPERT)) // ...create it
        {
         Print("The root folder of the Expert Advisor ..""\\"+NAME_EXPERT+"""\\ created successfully");
         //---
         return(root+"\\"); // Return the path to create a file for writing parameters of the Expert Advisor
        }
      else
        { Print("Error when creating the root folder of the Expert Advisor: ",ErrorDesc(Error())); return(""); }
     }
   else // If the folder of the Expert Advisor exists, 
     { return(root+"\\"); } // return the path to create a file for writing
//---
   return("");
  }
//+------------------------------------------------------------------+ 
//| WRITING DATA TO THE FILE                                         |
//+------------------------------------------------------------------+ 
void WriteReadParameters(string pth)
  {
   string nm_fl=pth+"ParametersOnTheFly.ini"; // File name and path
//---
// Get the file handle to read the file
   int hFl=FileOpen(nm_fl,FILE_READ|FILE_ANSI);
//---
   if(hFl!=INVALID_HANDLE) // If the handle has been obtained, the file exists
     {
      // Get parameters from the file
      if(!flgRead)
        {
         // Set the array size
         ArrayResize(arrParamIP,szArrIP);
         //---
         // Fill the array with values from the file
         flgRead=GetValuesParamsFromFile(hFl,arrParamIP);
        }
      //---
      // If the array size is correct,  ...
      if(ArraySize(arrParamIP)==szArrIP)
        {
         // ...set the parameters to the variables
         //---
         // Check the Indicator Period for correctness
         if((int)arrParamIP[0]<=0) { lcheck=10; }
         else { lcheck=(int)arrParamIP[0]; }
         gPeriod_Ind=(int)lcheck;
         //---
         gTakeProfit=arrParamIP[1];
         gStopLoss=arrParamIP[2];
         gReverse=arrParamIP[3];
         //---
         // Check the Lot for correctness
         if(arrParamIP[4]<=0)
           { lcheck=0.1; }
         else { lcheck=arrParamIP[4]; }
         gLot=lcheck;
        }
     }
   else // If the file does not exist
     {
      iZeroMemory(); // Zero out the variables
      //---
      // When creating the file, write current parameters of the Expert Advisor
      //---
      // Get the file handle to write to the file
      int hFl2=FileOpen(nm_fl,FILE_WRITE|FILE_CSV|FILE_ANSI,"");
      //---
      if(hFl2!=INVALID_HANDLE) // If the handle has been obtained
        {
         string sep="=";
         //---
         // Parameter names and values are obtained from arrays in the ARRAYS.mqh file
         for(int i=0; i<szArrIP; i++)
           { FileWrite(hFl2,arr_nmparams[i],sep,arr_vparams[i]); }
         //---
         FileClose(hFl2); // Close the file
         //---
         Print("The file with parameters of the "+NAME_EXPERT+" Expert Advisor created successfully.");
        }
     }
//---
   FileClose(hFl); // Close the file
  }
//+------------------------------------------------------------------+ 
//| READING PARAMETERS FROM THE FILE AND PLACING THEM IN THE ARRAY   |
//+------------------------------------------------------------------+ 
bool GetValuesParamsFromFile(int hdl,double &array[])
  {
   int
   sgn=0,  // Sign number "="
   cnts=0; // String counter
   string str=""; // String that is read
   ulong tell_seek=0,tseek=0; // Position of the pointer
   string delUser="------\nThe user has deleted the program from the chart!";
//---
   FileSeek(hdl,0,SEEK_SET); // Move the pointer to the beginning
//---
// Read until the file pointer reached the end of the file
   while(!FileIsEnding(hdl))
     {
      if(IsStopped()) // If the user has deleted the program
        {
         // Delete all objects created by the Expert Advisor
         DeleteAllExpertObjects();
         Print(delUser); return(false);
        }
      //---
      while(!FileIsLineEnding(hdl)) // Read until the end of the file
        {
         if(IsStopped()) // If the user has deleted the program
           {
            // Delete all objects created by the Expert Advisor
            DeleteAllExpertObjects();
            Print(delUser); return(false);
           }
         //---
         str=FileReadString(hdl); // Read the string
         //---
         sgn=StringFind(str,"=",0); // Get the separator number
         str=StringSubstr(str,sgn+1); // Get everything that is after the separator
         array[cnts]=SD(str); // Place the value in the array having converted it to the double type
         //---
         tell_seek=FileTell(hdl); // Get the position of the pointer
         //---
         if(FileIsLineEnding(hdl)) // If it is the end of the string
           {
            // Go to the next string if it is not the end of the file
            if(!FileIsEnding(hdl))
              { tseek=tell_seek+1; } // Increment the counter for the pointer
            //---
            // Move the pointer
            FileSeek(hdl,tseek,SEEK_SET); cnts++; break;
           }
        }
      //---
      // If it is the end of the file, exit the loop
      if(FileIsEnding(hdl)) { break; }
     }
//---
// Return that reading has been completed successfully
   return(true);
  }
//+------------------------------------------------------------------+ 
//| READING DATA FROM THE PANEL AND WRITING IT TO THE FILE           |
//+------------------------------------------------------------------+ 
void WriteNewData()
  {
// Get data from objects on the panel
   string getStr=""; // Object text
   string arrTemp[szArrIP]={""}; // Array of panel parameter values
//---
   for(int i=0; i<szArrIP; i++)
     {
      getStr=ObjectGetString(0,arr_objip[i][2],OBJPROP_TEXT);
      arrTemp[i]=FormatStringFromIP(getStr);
     }
//---
// Check if there is a file for writing/reading parameters to/from the file
// and write the new data, if available
   string lpath="";
//---
   if((lpath=CheckCreateGetPath())!="") // If the path has been obtained
     {
      string nm_fl=lpath+"ParametersOnTheFly.ini"; // File name
      //---
      int hFl=FileOpen(nm_fl,FILE_WRITE|FILE_CSV|FILE_ANSI,"");
      //---
      if(hFl!=INVALID_HANDLE) // If the handle has been obtained
        {
         string sep="=";
         //---
         for(int i=0; i<szArrIP; i++)
           { FileWrite(hFl,arr_nmparams[i],sep,arrTemp[i]); }
         //---
         FileClose(hFl); // Close the file
        }
     }
  }
//+------------------------------------------------------------------+ 
